# Maze Runner by Al Sweigart al@inventwithpython.com

# TODO - maze files are generated by mazemaker*.py programs.

import sys, os

# Maze file constants:
WALL = '#'
EMPTY = ' '
START = 'S'
EXIT = 'E'

FACE = chr(9786)
BLOCK = chr(9608)

print('MAZE RUNNER')
print('By Al Sweigart al@inventwithpython.com')
print()
print('(Maze files are generated by mazemaker.py)')

# Get the maze file's filename from the user:
while True:
    print('Enter the filename of the maze (or QUIT):')
    filename = input()

    if filename.upper() == 'QUIT':
        sys.exit()

    if os.path.exists(filename):
        break
    print('There is no file named', filename)

# Load the maze from a file:
mazeFile = open(filename)
maze = {}
lines = mazeFile.readlines()
playerx = None
playery = None
exitx = None
exity = None
y = 0
for line in lines:
    WIDTH = len(line.rstrip())
    for x, character in enumerate(line.rstrip()):
        assert character in (WALL, EMPTY, START, EXIT), f'Invalid character at column {x + 1}, line {y + 1}'
        if character in (WALL, EMPTY):
            maze[(x, y)] = character
        elif character == START:
            playerx, playery = x, y
            maze[(x, y)] = EMPTY
        elif character == EXIT:
            exitx, exity = x, y
            maze[(x, y)] = EMPTY
    y += 1
HEIGHT = y

assert playerx != None and playery != None, 'Missing start point in maze file.'
assert exitx != None and exity != None, 'Missing exit point in maze file.'

while True: # Main program loop.
    # Display the maze:
    for y in range(HEIGHT):
        for x in range(WIDTH):
            if (x, y) == (playerx, playery):
                print(FACE, end='')
            elif (x, y) == (exitx, exity):
                print('X', end='')
            elif maze[(x, y)] == WALL:
                print(BLOCK, end='')
            else:
                print(maze[(x, y)], end='')
        print() # Print a newline after printing the row.

    while True: # Get user move.
        print('                           W')
        print('Enter direction, or QUIT: ASD')
        move = input().upper()

        if move == 'QUIT':
            print('Thanks for playing!')
            sys.exit()

        if move not in ['W', 'A', 'S', 'D']:
            print('Invalid direction. Enter one of W, A, S, or D.')
            continue

        # Check if the player can move in that direction:
        if move == 'W' and maze[(playerx, playery - 1)] == EMPTY:
                break
        elif move == 'S' and maze[(playerx, playery + 1)] == EMPTY:
                break
        elif move == 'A' and maze[(playerx - 1, playery)] == EMPTY:
                break
        elif move == 'D' and maze[(playerx + 1, playery)] == EMPTY:
                break

        print('You cannot move in that direction.')

    # At this point, move contains a valid direction to move in.

    # Keep moving in this direction until you encounter a branch point.
    if move == 'W':
        while True:
            playery -= 1
            if (playerx, playery) == (exitx, exity):
                break
            if maze[(playerx, playery - 1)] == WALL:
                break # Break if we've hit a wall in the direction we were traveling.
            if maze[(playerx - 1, playery)] == EMPTY or maze[(playerx + 1, playery)] == EMPTY:
                break # Break if we've reached a branch point.
    elif move == 'S':
        while True:
            playery += 1
            if (playerx, playery) == (exitx, exity):
                break
            if maze[(playerx, playery + 1)] == WALL:
                break # Break if we've hit a wall in the direction we were traveling.
            if maze[(playerx - 1, playery)] == EMPTY or maze[(playerx + 1, playery)] == EMPTY:
                break # Break if we've reached a branch point.
    elif move == 'A':
        while True:
            playerx -= 1
            if (playerx, playery) == (exitx, exity):
                break
            if maze[(playerx - 1, playery)] == WALL:
                break # Break if we've hit a wall in the direction we were traveling.
            if maze[(playerx, playery - 1)] == EMPTY or maze[(playerx, playery + 1)] == EMPTY:
                break # Break if we've reached a branch point.
    elif move == 'D':
        while True:
            playerx += 1
            if (playerx, playery) == (exitx, exity):
                break
            if maze[(playerx + 1, playery)] == WALL:
                break # Break if we've hit a wall in the direction we were traveling.
            if maze[(playerx, playery - 1)] == EMPTY or maze[(playerx, playery + 1)] == EMPTY:
                break # Break if we've reached a branch point.

    if (playerx, playery) == (exitx, exity):
        # TODO - display the maze one last time.
        print('You have reached the exit! Good job!')
        print('Thanks for playing!')
        sys.exit()
